﻿/*
 * This will convert email addresses in the clipboard from the Outlook format, e.g. John Doe <John.Doe@company.com>; "Steve Smith" <Steve.Smith@company.com> to regular emails, e.g. John.Doe@company.com, Steve.Smith@company.com.
 * This code is distributed under the terms of the GNU General Public License.
*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.Text;
using System.IO;

namespace Outlook_Email_Address_Converter
{
    static class Program
    {
        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        /// 

        public static string InstallDir = Path.GetDirectoryName(Application.ExecutablePath);
        public static string SettingsINI = InstallDir + "\\settings.ini";
        public static string program_name = "Outlook Email Address Converter";
        public static string global_first_separator = "Comma";
        public static string global_second_separator = "New line";
        public static bool ran_silently_once = false;

        [STAThread]
        static void Main()
        {
            // if we don't have a SettingsINI file yet, let's create one.
            if (File.Exists(SettingsINI) == false)
            {
                try
                {
                    File.WriteAllText(SettingsINI, "");
                }
                catch (Exception e)
                {
                    MessageBox.Show("An error occurred while writing to the " + SettingsINI + " file. " + e.Message);
                }
                WriteINI("settings", "first_separator", "Comma", SettingsINI);
                WriteINI("settings", "second_separator", "New line", SettingsINI);
                WriteINI("settings", "ran_silently_once", "0", SettingsINI);
            }
            else
            {
                global_first_separator = ReadINI("settings", "first_separator", SettingsINI);
                global_second_separator = ReadINI("settings", "second_separator", SettingsINI);
                if (ReadINI("settings", "ran_silently_once", SettingsINI) == "1")
                {
                    ran_silently_once = true;
                }
                else
                {
                    ran_silently_once = false;
                }
            }

            // if the user copied the email addresses to the clipboard, we need to convert them without showing the form.
            string clipboard_text = Clipboard.GetText();
            if (clipboard_text.Length == 0 || clipboard_text.IndexOf("@") == -1 || clipboard_text.IndexOf("<") == -1 || clipboard_text.IndexOf(">") == -1)
            {
                // the user either has nothing copied in the clipboard or some text other than outlook email addresses. Show the form.
                Application.EnableVisualStyles();
                Application.SetCompatibleTextRenderingDefault(false);
                Application.Run(new Form1());
            }
            else
            {
                // the user has outlook email addresses copied to the clipboard. Convert them silently. Then exit the application.
                ConvertEmails(clipboard_text, global_first_separator, global_second_separator, true);
                Application.Exit();
            }
        }

        public static string ConvertEmails(string text, string first_separator, string second_separator = "", bool from_clipboard = true)
        {
            // first let's validate that the text contains at least one email address between < and >
            int find_first_left_arrow = text.IndexOf("<");
            int find_first_right_arrow = text.IndexOf(">");
            if (find_first_left_arrow == -1 || find_first_right_arrow < find_first_left_arrow)
            {
                if (from_clipboard)
                {
                    MessageBox.Show("The text in the clipboard does not appear to contain Outlook email addresses. The expected format is:" + Environment.NewLine + Environment.NewLine + "\"John Doe\" <John.Doe@company.com>; \"Steve Smith\" <Steve.Smith@company.com>" + Environment.NewLine + Environment.NewLine + "Please double-check the format and try again.", program_name);
                    Application.Exit();
                }
                else
                {
                    MessageBox.Show("The text does not appear to contain Outlook email addresses. The expected format is:" + Environment.NewLine + Environment.NewLine + "\"John Doe\" <John.Doe@company.com>; \"Steve Smith\" <Steve.Smith@company.com>" + Environment.NewLine + Environment.NewLine + "Please double-check the format and try again.", program_name);
                }
                return "error_shown";
            }
            if (from_clipboard && ran_silently_once == false)
            {
                DialogResult result;
                result = MessageBox.Show("It appears that you have email addresses copied to the clipboard. If you continue, this utility will reformat these email addresses, save them back to the clipboard, and then it will close, allowing you to paste the reformatted email addresses in your application. Do you want to continue?" + Environment.NewLine + Environment.NewLine + "Going forward, if you already have email addresses copied to the clipboard when you run this utility, it will convert them silently. If you do not have email addresses copied to the clipboard, it will load the main window to let you configure options.", "Outlook Email Address Converter", MessageBoxButtons.YesNo);
                if (result == DialogResult.No)
                {
                    Application.Exit();
                    return "error_shown";
                }
                else
                {
                    // flag this in the INI so that this warning does not appear every time.
                    ran_silently_once = true;
                    WriteINI("settings", "ran_silently_once", "1", SettingsINI);
                }
            }
            if (first_separator.ToLower() == "comma")
                first_separator = ",";
            else if (first_separator.ToLower() == "semicolon")
                first_separator = ";";
            else if (first_separator.ToLower() == "space")
                first_separator = " ";
            else if (first_separator.ToLower() == "new line")
                first_separator = Environment.NewLine;
            if (second_separator.ToLower() == "comma")
                second_separator = ",";
            else if (second_separator.ToLower() == "semicolon")
                second_separator = ";";
            else if (second_separator.ToLower() == "space")
                second_separator = " ";
            else if (second_separator.ToLower() == "new line")
                second_separator = Environment.NewLine;
            else if (second_separator.ToLower() == "none")
                second_separator = "";
            int cursor = 0;
            List<string> email_addresses = new List<string>();
            for (int i = 0; i < 100000; i++)
            {
                // search for the < character.
                int find_left_arrow = text.IndexOf("<", cursor);
                if (find_left_arrow == -1)
                {
                    // no more arrows. We should have everything in our list.
                    break;
                }
                else
                {
                    cursor = find_left_arrow + 1;
                }
                int find_right_arrow = text.IndexOf(">", find_left_arrow);
                if (find_right_arrow == -1)
                {
                    MessageBox.Show("A starting < character was found, but the closing character, >, was not found. Please make sure that the email addresses were specified correctly.", program_name);
                    break;
                }
                // grab the email address between < and >. Then add it our list.
                string this_email = text.Substring(find_left_arrow + 1, find_right_arrow - find_left_arrow - 1);
                email_addresses.Add(this_email);
            }
            // now that we have our emails in our list, let's format them.
            string final_text = "";
            int item_cursor = 0;
            foreach (string this_email in email_addresses)
            {
                item_cursor++;
                final_text += this_email;
                // if we are not on our last one, add the separators.
                if (item_cursor < email_addresses.Count)
                {
                    final_text += first_separator + second_separator;
                }
            }
            if (from_clipboard)
            {
                // we are processing data in the clipboard. Let's copy the new text to the clipboard.
                Clipboard.Clear();
                Clipboard.SetText(final_text);
                return "text_copied";
            }
            else
            {
                // we are processing data from the text field. Just return the final text, and the other function will process it.
                return final_text;
            }
        }

        [DllImport("kernel32")]
        public static extern int GetPrivateProfileString(string section, string key, string def, StringBuilder retVal, int size, string filePath);

        [DllImport("KERNEL32.DLL", EntryPoint = "WritePrivateProfileStringW",
          SetLastError = true,
          CharSet = CharSet.Unicode, ExactSpelling = true,
          CallingConvention = CallingConvention.StdCall)]
        public static extern int WritePrivateProfileString(
          string lpAppName,
          string lpKeyName,
          string lpString,
          string lpFilename);

        public static string ReadINI(string Section, string Key, string thePath)
        {
            StringBuilder temp = new StringBuilder(255);
            int i = GetPrivateProfileString(Section, Key, "", temp, 255, thePath);
            return temp.ToString();
        }

        public static int ReadIntFromINI(string Section, string Key, string thePath, int set_default = -1)
        {
            // this will read an integer from an INI file. This function allows the option to 
            // set a default value to insert in case the value is currently blank in the INI.
            StringBuilder temp = new StringBuilder(255);
            int i = GetPrivateProfileString(Section, Key, "", temp, 255, thePath);
            string the_text = temp.ToString();
            try
            {
                return Convert.ToInt32(the_text);
            }
            catch
            {
                if (set_default != -1)
                    WriteINI(Section, Key, set_default.ToString(), thePath);
                else
                    set_default = 0;

                return set_default;
            }

        }

        public static int WriteINI(string Section, string Key, string Value, string thePath)
        {
            int try_once;
            try
            {
                try_once = WritePrivateProfileString(Section, Key, Value, thePath);
                if (try_once == 0)
                    return 0;
                else
                    return 1;
            }
            catch
            {
                return 0;
            }
        }
    }
}
